import json
import sys


class Bot:
    def __init__(self, actor_id):
        self.actor_id = actor_id

    def react(self, events: str) -> str:
        events = json.loads(events)
        assert len(events) > 0

        # 最後のイベントの `type` によって分岐
        if events[-1]["type"] == "tsumo":  # type: ignore
            # 最後にツモった牌を捨てる
            return json.dumps({
                "type": "dahai",
                "pai": events[-1]["pai"],  # type: ignore
                "actor": self.actor_id,
                "tsumogiri": True,
            }, separators=(",", ":"))
        else:
            # それ以外
            # 他人の打牌に対して常にアクションしない
            return json.dumps({
                "type": "none",
            }, separators=(",", ":"))


def main():
    player_id = int(sys.argv[1])
    assert player_id in range(4)
    bot = Bot(player_id)

    while True:
        line = sys.stdin.readline().strip()
        resp = bot.react(line)
        sys.stdout.write(resp + "\n")
        sys.stdout.flush()


def test_react():
    player_id = 1
    bot = Bot(player_id)

    # イベントのリストを文字列として受け取り、単一のイベントを文字列として返す
    resp = bot.react('[{"type":"start_game","id":1}]')
    assert resp == '{"type":"none"}'

    resp = bot.react('[{"type":"start_kyoku","bakaze":"E","kyoku":1,"honba":0,"kyotaku":0,"oya":0,"dora_marker":"7s","tehais":[["?","?","?","?","?","?","?","?","?","?","?","?","?"],["3m","4m","3p","5pr","7p","9p","4s","4s","5sr","7s","7s","W","N"],["?","?","?","?","?","?","?","?","?","?","?","?","?"],["?","?","?","?","?","?","?","?","?","?","?","?","?"]]},{"type":"tsumo","actor":0,"pai":"?"},{"type":"dahai","actor":0,"pai":"6s","tsumogiri":false},{"type":"tsumo","actor":1,"pai":"3m"}]')
    assert resp == '{"type":"dahai","pai":"3m","actor":1,"tsumogiri":true}'


if __name__ == "__main__":
    main()
